/* Target-dependent code for the Samsung CalmRISC16.
   Copyright 2000, Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include "defs.h"
#include "inferior.h"
#include "symfile.h"		/* for entry_point_address */
#include "gdbcore.h"
#include "arch-utils.h"

/* gdb register numbers.  */
#define CALMRISC16_GR_REGNUM_LO	 0
#define CALMRISC16_GR_REGNUM_HI	15
#define CALMRISC16_ER_REGNUM_LO	16
#define CALMRISC16_ER_REGNUM_HI	23
#define CALMRISC16_AR_REGNUM_LO	24
#define CALMRISC16_AR_REGNUM_HI	31

/* Register numbers by index.  */
#define CALMRISC16_GR_REGNUM(grnum) (CALMRISC16_GR_REGNUM_LO + (grnum))
#define CALMRISC16_ER_REGNUM(ernum) (CALMRISC16_ER_REGNUM_LO + (ernum) - 8)
#define CALMRISC16_AR_REGNUM(arnum) (CALMRISC16_AR_REGNUM_LO + (arnum) - 8)

/* Specific register numbers.  */
#define CALMRISC16_LR_REGNUM       (CALMRISC16_AR_REGNUM (14)) /* LR is a14 */
#define CALMRISC16_SP_REGNUM       (CALMRISC16_AR_REGNUM (15)) /* SP is a15 */
#define CALMRISC16_PC_REGNUM	32
#define CALMRISC16_SPC_FIQ_REGNUM	33
#define CALMRISC16_SPC_IRQ_REGNUM	34
#define CALMRISC16_SR_REGNUM	35
#define CALMRISC16_SSR_FIQ_REGNUM	36
#define CALMRISC16_SSR_IRQ_REGNUM	37
#define CALMRISC16_SSR_SWI_REGNUM	38

/* Pseudo registers */
#define CALMRISC16_PSEUDO_LR_REGNUM 39

#define CALMRISC16_NUM_REGS 39
#define CALMRISC16_NUM_PSEUDO_REGS 1
#define CALMRISC16_ALL_REGS (CALMRISC16_NUM_REGS + CALMRISC16_NUM_PSEUDO_REGS)

static int pc_regnum = CALMRISC16_PC_REGNUM;
static int sp_regnum = CALMRISC16_SP_REGNUM;
static int fp_regnum = CALMRISC16_SP_REGNUM;

static LONGEST calmrisc16_call_dummy_words[] =
{0};

struct frame_extra_info
  {
    int framesize;			/* size of a stack frame. */
    int framereg;			/* register used for frame pointer
                                           in this frame */
    int frameoff;			/* displacement from frame pointer
                                           to caller's SP */

  };

/* The array of register names will be constructed and filled in the
   first time it is used.  */

static char **calmrisc16_register_names;

static char *
calmrisc16_register_name (int reg)
{
  int r;
  char buf[20];

  if (reg < 0)
    return "?toosmall?";
  if (reg >= CALMRISC16_ALL_REGS)
    return "?toolarge?";

  /* Construct the array of register names.  */
  if (calmrisc16_register_names == NULL)
    {
      calmrisc16_register_names = 
	(char **) xmalloc (CALMRISC16_ALL_REGS * sizeof (char *));

      for (r = CALMRISC16_GR_REGNUM_LO; r <= CALMRISC16_GR_REGNUM_HI; ++r)
	{
	  sprintf (buf, "r%d", r - CALMRISC16_GR_REGNUM_LO);
	  calmrisc16_register_names[r] = strsave (buf);
	}
      for (r = CALMRISC16_ER_REGNUM_LO; r <= CALMRISC16_ER_REGNUM_HI; ++r)
	{
	  sprintf (buf, "e%d", r - CALMRISC16_ER_REGNUM_LO + 8);
	  calmrisc16_register_names[r] = strsave (buf);
	}
      for (r = CALMRISC16_AR_REGNUM_LO; r <= CALMRISC16_AR_REGNUM_HI; ++r)
	{
	  sprintf (buf, "a%d", r - CALMRISC16_AR_REGNUM_LO + 8);
	  calmrisc16_register_names[r] = strsave (buf);
	}
      calmrisc16_register_names[CALMRISC16_PC_REGNUM] = "pc";
      calmrisc16_register_names[CALMRISC16_SPC_FIQ_REGNUM] = "spc_fiq";
      calmrisc16_register_names[CALMRISC16_SPC_IRQ_REGNUM] = "spc_irq";
      calmrisc16_register_names[CALMRISC16_SR_REGNUM] = "sr";
      calmrisc16_register_names[CALMRISC16_SSR_FIQ_REGNUM] = "ssr_fiq";
      calmrisc16_register_names[CALMRISC16_SSR_IRQ_REGNUM] = "ssr_irq";
      calmrisc16_register_names[CALMRISC16_SSR_SWI_REGNUM] = "ssr_swi";
      calmrisc16_register_names[CALMRISC16_PSEUDO_LR_REGNUM] = "lr";

    }
  return calmrisc16_register_names[reg];
}

static int
calmrisc16_register_raw_size (int reg)
{
  if ((CALMRISC16_AR_REGNUM_LO <= reg && reg <= CALMRISC16_AR_REGNUM_HI)
      || reg == CALMRISC16_PC_REGNUM
      || reg == CALMRISC16_SPC_FIQ_REGNUM
      || reg == CALMRISC16_SPC_IRQ_REGNUM
      || reg == CALMRISC16_PSEUDO_LR_REGNUM)
    return 4;
  else
    return 2;
}

static int
calmrisc16_register_virtual_size (int reg)
{
  if ((CALMRISC16_AR_REGNUM_LO <= reg && reg <= CALMRISC16_AR_REGNUM_HI)
      || reg == CALMRISC16_PC_REGNUM
      || reg == CALMRISC16_SPC_FIQ_REGNUM
      || reg == CALMRISC16_SPC_IRQ_REGNUM
      || reg == CALMRISC16_PSEUDO_LR_REGNUM)
    return 4;
  else
    return 2;
}

static struct type *
calmrisc16_register_virtual_type (int reg)
{
  static struct type *void_func_ptr = 0;

  if (void_func_ptr == 0)
    {
      struct type *void_ptr = lookup_pointer_type (builtin_type_void);
      void_func_ptr = 
	lookup_pointer_type (lookup_function_type (builtin_type_void));
    }

  if (CALMRISC16_AR_REGNUM_LO <= reg && reg <= CALMRISC16_AR_REGNUM_HI)
    return builtin_type_ptr;
  else if (reg == CALMRISC16_PC_REGNUM || reg == CALMRISC16_PSEUDO_LR_REGNUM)
    return void_func_ptr;
  else
    return builtin_type_int;
}

static int
calmrisc16_register_byte (int reg)
{
  return (reg * 4);
}

/* Convert an instruction pointer to an address */
static CORE_ADDR
calmrisc16_insn_pointer_to_address (CORE_ADDR ptr)
{
  return ptr | 0x400000;
}

/* Convert an instruction address to a pointer */
static CORE_ADDR
calmrisc16_insn_address_to_pointer (CORE_ADDR addr)
{
  return addr & 0x03fffff;
}

static CORE_ADDR
calmrisc16_read_pc (int pid)
{
  CORE_ADDR pc_value = read_register_pid (CALMRISC16_PC_REGNUM, pid);
  return calmrisc16_insn_pointer_to_address (pc_value);
}

static void
calmrisc16_write_pc (CORE_ADDR new_pc, int pid)
{
  write_register_pid (CALMRISC16_PC_REGNUM,
                      calmrisc16_insn_address_to_pointer (new_pc),
		      pid);
}

static int
calmrisc16_register_convertible (int nr)
{
#if 0
  return nr == CALMRISC16_PC_REGNUM || nr == CALMRISC16_LR_REGNUM
      || nr == CALMRISC16_PSEUDO_LR_REGNUM;
#else
  return 0;
#endif
}

static void
calmrisc16_register_convert_to_virtual (int regnum, struct type *type,
                                        char *from, char *to)
{
  /* We don't list the CALMRISC16_LR_REGNUM case here because we
     prefer to represent it without the insn mapping.  */
  if (regnum == CALMRISC16_PC_REGNUM || regnum == CALMRISC16_PSEUDO_LR_REGNUM)
    {
      CORE_ADDR ptr = extract_address (from, REGISTER_RAW_SIZE (regnum));

      store_address (to, REGISTER_VIRTUAL_SIZE (regnum),
                     calmrisc16_insn_pointer_to_address (ptr));
    }
  else
    {
      /* just copy the bytes */
      memcpy (to, from, REGISTER_RAW_SIZE (regnum));
    }
}

static void
calmrisc16_register_convert_to_raw (struct type *type, int regnum,
                                    char *from, char *to)
{
  if (regnum == CALMRISC16_PC_REGNUM || regnum == CALMRISC16_LR_REGNUM
      || regnum == CALMRISC16_PSEUDO_LR_REGNUM)
    {
      CORE_ADDR addr = extract_address (from, REGISTER_VIRTUAL_SIZE (regnum));

      store_address (to, REGISTER_RAW_SIZE (regnum),
                     calmrisc16_insn_address_to_pointer (addr));
    }
  else
    {
      /* just copy the bytes */
      memcpy (to, from, REGISTER_RAW_SIZE (regnum));
    }
}

static CORE_ADDR
calmrisc16_pointer_to_address (struct type *type, void *buf)
{
  enum type_code target = TYPE_CODE (TYPE_TARGET_TYPE (type));
  CORE_ADDR addr = extract_address (buf, TYPE_LENGTH (type));

  if (target == TYPE_CODE_FUNC || target == TYPE_CODE_METHOD)
    addr = calmrisc16_insn_pointer_to_address (addr);

  return addr;
}

static void
calmrisc16_address_to_pointer (struct type *type, void *buf, CORE_ADDR addr)
{
  enum type_code target = TYPE_CODE (TYPE_TARGET_TYPE (type));

  if (target == TYPE_CODE_FUNC || target == TYPE_CODE_METHOD)
    addr = calmrisc16_insn_address_to_pointer (addr);

  store_address (buf, TYPE_LENGTH (type), addr);
}

static unsigned char *
calmrisc16_breakpoint_from_pc (CORE_ADDR *pcptr, int *lenp)
{
  static unsigned char breakpoint[] = {0x9e, 0x41};
  *lenp = sizeof (breakpoint);
  return breakpoint;
}

/* On CalmRisc16, we've set the frame pointer to be equal to
   the stack pointer since there is no requirement to have
   a frame pointer register.  Thus, the frame_chain operation
   should return the current sp in the previous frame.  In cases
   where there is a register used for a frame pointer, we will
   use it to return the correct value.  */

static CORE_ADDR
calmrisc16_frame_chain (struct frame_info *frame)
{
  if (PC_IN_CALL_DUMMY (frame->pc, frame->frame, frame->frame))
    return frame->frame;
  else
    {
      FRAME_INIT_SAVED_REGS (frame);
      if (frame->extra_info->framereg != CALMRISC16_SP_REGNUM)
	{
	  char *fr_raw_buffer = alloca (MAX_REGISTER_RAW_SIZE);
	  CORE_ADDR fr_val;
	  int framereg;

	  framereg = frame->extra_info->framereg;
	  get_saved_register (fr_raw_buffer, NULL, NULL, frame, framereg, NULL);
	  fr_val = extract_address (fr_raw_buffer,
	                            REGISTER_RAW_SIZE (framereg));
	  return fr_val + frame->extra_info->frameoff;
	}
      else
	return frame->frame + frame->extra_info->framesize;
    }
}

static CORE_ADDR
calmrisc16_saved_pc_after_call (struct frame_info *frame)
{
  return
    calmrisc16_insn_pointer_to_address (read_register (CALMRISC16_LR_REGNUM));
}

static CORE_ADDR
calmrisc16_frame_saved_pc (struct frame_info *frame)
{
  CORE_ADDR fspc;

  if (PC_IN_CALL_DUMMY (frame->pc, frame->frame, frame->frame))
    return calmrisc16_insn_pointer_to_address (
             generic_read_register_dummy (frame->pc, frame->frame, pc_regnum));
  else
    {
      FRAME_INIT_SAVED_REGS (frame);
      if (frame->saved_regs[CALMRISC16_LR_REGNUM])
	{
	  fspc = read_memory_integer (frame->saved_regs[CALMRISC16_LR_REGNUM], 
				      REGISTER_RAW_SIZE (CALMRISC16_LR_REGNUM));
	  fspc = calmrisc16_insn_pointer_to_address (fspc);
	}
      else
	fspc = calmrisc16_saved_pc_after_call (frame);

      return fspc;
    }
}

static CORE_ADDR
calmrisc16_frame_args_address (struct frame_info *frame)
{
  FRAME_INIT_SAVED_REGS (frame);
  if (frame->extra_info->framereg != CALMRISC16_SP_REGNUM)
    {
      char *fr_raw_buffer = alloca (MAX_REGISTER_RAW_SIZE);
      CORE_ADDR fr_val;
      int framereg;

      framereg = frame->extra_info->framereg;
      get_saved_register (fr_raw_buffer, NULL, NULL, frame, framereg, NULL);
      fr_val = extract_address (fr_raw_buffer, REGISTER_RAW_SIZE (framereg));
      return fr_val;
    }
  else
    return frame->frame;
}

static CORE_ADDR
calmrisc16_frame_locals_address (struct frame_info *frame)
{
  return calmrisc16_frame_args_address (frame);
}

/* Fetch the instruction at ADDR; return the length of the instruction
   in bytes.  The first 16-bit word of the instruction is stored at
   INSTR.  The second 16-bit word of the instruction, if any, is
   stored at WORD2.  The total length of the instruction is returned;
   0 is returned if there was an error fetching the instruction.  (In
   other words, the expected return values from this function are 0,
   for an error, 2, for a 1-word instruction, and 4, for a 2-word
   instruction.)  */

static int
fetch_instruction (CORE_ADDR addr, long *instr, long *word2)
{
  char buf[2];
  int status;

  status = target_read_memory (addr, buf, 2);
  if (status != 0)
    return 0;
  *instr = extract_unsigned_integer (buf, 2);

  if (((*instr & 0xf0f0) == 0x80f0) || ((*instr & 0xffc0) == 0x9f00))
    {
      status = target_read_memory (addr + 2, buf, 2);
      if (status != 0)
	return 0;
      *word2 = extract_signed_integer (buf, 2);
      return 4;
    }
  else
    return 2;
}

/* Scan a CalmRISC16 prologue, starting at PC.
   If FRAME is non-zero, fill in its saved_regs with appropriate addresses.
   We assume FRAME's saved_regs array has already been allocated and cleared.
   Return the first PC value after the prologue.  */

static CORE_ADDR
calmrisc16_analyze_prologue (CORE_ADDR pc, CORE_ADDR lim_pc,
                             struct frame_info *frame)
{
  int framesize = 0;		/* total size of frame */
  int framereg  = -1;		/* register, if any used for frame pointer */
  int frameoff  = 0;		/* offset from frame pointer to caller's sp */
  char *reg_saved = alloca (CALMRISC16_NUM_REGS * sizeof (char));
  int *reg_offset = alloca (CALMRISC16_NUM_REGS * sizeof (int));

  memset (reg_saved, 0, CALMRISC16_NUM_REGS * sizeof (char));
  
  while (pc < lim_pc || lim_pc == 0)
    {
      long instr, word2;
      int ilen;

      ilen = fetch_instruction (pc, &instr, &word2);
      if (ilen == 0)
	break;

      /* PUSH Rn,Rm / PUSH Rn */
      if ((instr & 0xf0f8) == 0xa0f0)
	{
	  int Rm, Rn;
	  Rm = (instr & 0x0f00) >> 8;
	  Rn = (instr & 0x0007) | (Rm & 0x8);
	  reg_saved[CALMRISC16_GR_REGNUM_LO + Rn] = 1;
	  reg_offset[CALMRISC16_GR_REGNUM_LO + Rn] = -framesize;
	  framesize += 2;
	  if (Rn != Rm)
	    {
	      reg_saved[CALMRISC16_GR_REGNUM_LO + Rm] = 1;
	      reg_offset[CALMRISC16_GR_REGNUM_LO + Rm] = -framesize;
	      framesize += 2;
	    }
	}

      /* PUSH An,Am / PUSH An */
      else if ((instr & 0xf8f8) == 0xa8f8)
	{
	  int Am, An;
	  Am = (instr & 0x0700) >> 8;
	  An = (instr & 0x0007);
	  reg_saved[CALMRISC16_GR_REGNUM_LO + 8 + An] = 1;
	  reg_offset[CALMRISC16_GR_REGNUM_LO + 8 + An] = -framesize;
	  reg_saved[CALMRISC16_ER_REGNUM_LO + An] = 1;
	  reg_offset[CALMRISC16_ER_REGNUM_LO + An] = -framesize - 2;
	  reg_saved[CALMRISC16_AR_REGNUM_LO + An] = 1;
	  reg_offset[CALMRISC16_AR_REGNUM_LO + An] = -framesize - 2;
	  framesize += 4;
	  if (An != Am)
	    {
	      reg_saved[CALMRISC16_GR_REGNUM_LO + 8 + Am] = 1;
	      reg_offset[CALMRISC16_GR_REGNUM_LO + 8 + Am] = -framesize;
	      reg_saved[CALMRISC16_ER_REGNUM_LO + Am] = 1;
	      reg_offset[CALMRISC16_ER_REGNUM_LO + Am] = -framesize - 2;
	      reg_saved[CALMRISC16_AR_REGNUM_LO + Am] = 1;
	      reg_offset[CALMRISC16_AR_REGNUM_LO + Am] = -framesize - 2;
	      framesize += 4;
	    }
	}

      /* SUB SP, #<imm:16> */
      else if ((instr & 0xffff) == 0x81ff)
	{
	  int disp = word2 & 0xffff;

	  framesize += disp;
	}

      /* SUB SP, #<imm:5> */
      else if ((instr & 0xffe0) == 0xaf60)
	{
	  int disp = instr & 0x1f;

	  framesize += disp;
	}

      /* LD An, SP */
      else if ((instr & 0xf8ff) == 0xa81f)
        {
	  int An = (instr & 0x0700) >> 8;

	  framereg = CALMRISC16_AR_REGNUM_LO + An;
	  frameoff = framesize;
	}

      /* ADD An, #<imm:16>, when An is the frame register */
      else if (framereg != -1
        && (instr & 0xffff) == (0x81f0 | (framereg - CALMRISC16_AR_REGNUM_LO)))
	{
	  int disp = word2 & 0xffff;

	  frameoff -= disp;
	}

      /* ADD An, #<imm:5>, when An is the frame register */
      else if (framereg != -1
        && (instr & 0xffe0) 
	      == (0xa840 | ((framereg - CALMRISC16_AR_REGNUM_LO) << 8)))
	{
	  int disp = instr & 0x001f;

	  frameoff -= disp;
	}

      /* Not recognized, so get out... */
      else
	{
	  break;
	}

      pc += ilen;
    }

  if (frame)
    {
      int i;
      CORE_ADDR fp;

      if (framereg == -1)
	{
	  framereg = CALMRISC16_SP_REGNUM;
	  frameoff = framesize;
	  fp = frame->frame;
	}
      else
        {
	  char *fr_raw_buffer = alloca (MAX_REGISTER_RAW_SIZE);
	  get_saved_register (fr_raw_buffer, NULL, NULL, frame, framereg, NULL);
	  fp = extract_address (fr_raw_buffer, REGISTER_RAW_SIZE (framereg));
	}

      for (i = 0; i < CALMRISC16_NUM_REGS; i++)
	if (reg_saved[i])
	  frame->saved_regs[i] = fp + frameoff + reg_offset[i];

      frame->extra_info->framesize = framesize;
      frame->extra_info->framereg = framereg;
      frame->extra_info->frameoff = frameoff;
    }
  return pc;
}


static CORE_ADDR
calmrisc16_skip_prologue (CORE_ADDR pc)
{
  return calmrisc16_analyze_prologue (pc, 0, 0);
}

static void
calmrisc16_frame_init_saved_regs (struct frame_info *frame)
{
  if (frame->saved_regs)
    return;

  frame_saved_regs_zalloc (frame);

  frame->saved_regs[fp_regnum] = frame->frame;

  /* Find the beginning of this function, so we can analyze its
     prologue.  */     
  {
    CORE_ADDR func_addr, func_end;

    if (find_pc_partial_function (frame->pc, NULL, &func_addr, &func_end))
      calmrisc16_analyze_prologue (func_addr, frame->pc, frame);
  }
}

/* Should we use EXTRACT_STRUCT_VALUE_ADDRESS instead of
   EXTRACT_RETURN_VALUE?  GCC_P is true if compiled with gcc
   and TYPE is the type (which is known to be struct, union or array).

   The calmrisc16 returns all structs in memory.  */

static int
calmrisc16_use_struct_convention (int gcc_p, struct type *type)
{
  return 1;
}

static void
calmrisc16_extract_return_value (struct type *type, char *regbuf, char *valbuf)
{
  int i;
  int regno;
  int len = TYPE_LENGTH (type);

  for (i = 0, regno = CALMRISC16_GR_REGNUM (0); i < len ; i += 2, regno += 1)
    {
      int rsize = REGISTER_RAW_SIZE (regno);
      int rlen = (len - i > rsize ? rsize : len - i);
      int roff = (rlen < rsize ? rsize - rlen : 0);
      memcpy (valbuf + i, regbuf + calmrisc16_register_byte (regno) + roff,
              rlen);
    }
}

static CORE_ADDR
calmrisc16_extract_struct_value_address (char *regbuf)
{
  int len = REGISTER_RAW_SIZE (CALMRISC16_GR_REGNUM (0))
            + REGISTER_RAW_SIZE (CALMRISC16_GR_REGNUM (1));
  char *buf = alloca (len);

  memcpy (buf,
          regbuf + calmrisc16_register_byte (CALMRISC16_GR_REGNUM (0)),
          REGISTER_RAW_SIZE (CALMRISC16_GR_REGNUM (0)));
  memcpy (buf + REGISTER_RAW_SIZE (CALMRISC16_GR_REGNUM (0)),
          regbuf + calmrisc16_register_byte (CALMRISC16_GR_REGNUM (1)),
	  REGISTER_RAW_SIZE (CALMRISC16_GR_REGNUM (1)));

  return extract_address (buf, len);
}

static void
calmrisc16_store_struct_return (CORE_ADDR addr, CORE_ADDR sp)
{
  write_register (CALMRISC16_GR_REGNUM (0), (addr >> 16) & 0xffff);
  write_register (CALMRISC16_GR_REGNUM (1), addr & 0xffff);
}

static int
calmrisc16_frameless_function_invocation (struct frame_info *frame)
{
  FRAME_INIT_SAVED_REGS (frame);
  return (frame->extra_info->framesize == 0);
}

static void
calmrisc16_init_extra_frame_info (int fromleaf, struct frame_info *frame)
{
  frame->extra_info = (struct frame_extra_info *)
    frame_obstack_alloc (sizeof (struct frame_extra_info));
  frame->extra_info->framesize = 0;
  frame->extra_info->framereg = 0;
  frame->extra_info->frameoff = 0;
}

/* Stack pointer points 1 word before actual contents of top of stack.  */
#define SP_BIAS 2

/* Push arguments on the inferior's stack in preparation for doing
   an inferior function call.  The number of arguments to push is
   given by NARGS; the actual arguments to push are given by ARGS;
   the initial stack pointer is given by SP.  STRUCT_RETURN will
   be non-zero if the function to be called will return a structure
   and STRUCT_ADDR gives the address of the area into which the
   structure should be stored.  The new stack pointer is returned.  */

static CORE_ADDR
calmrisc16_push_arguments (int nargs, value_ptr *args, CORE_ADDR sp,
		    int struct_return, CORE_ADDR struct_addr)
{
  int i;

  sp -= SP_BIAS;	/* Just in case if the rest of gdb doesn't
                           know about the stack bias... */

  for (i = nargs - 1; i >= 0; i--)
    {
      int len = TYPE_LENGTH (VALUE_ENCLOSING_TYPE (args[i]));
      int container_len = ((len + 16 / TARGET_CHAR_BIT - 1)
                           & ~ (16 / TARGET_CHAR_BIT - 1));
      int offset = len < 2 ? container_len - len : 0;

      sp -= container_len;
      write_memory (sp + offset + SP_BIAS, VALUE_CONTENTS_ALL (args[i]), len);
    }

  if (struct_return)
    {
      int size = TARGET_PTR_BIT / TARGET_CHAR_BIT;
      char *buf = alloca (size);

      store_address (buf, size, struct_addr);
      sp -= size;
      write_memory (sp + SP_BIAS, buf, size);
    }

  return sp;
}

static CORE_ADDR
calmrisc16_push_return_address (CORE_ADDR pc, CORE_ADDR sp)
{
  write_register (CALMRISC16_LR_REGNUM, CALL_DUMMY_ADDRESS ());
  return sp;
}

static void
calmrisc16_store_return_value (struct type *type, char *valbuf)
{
  int regno;
  int len;
  int off;

  len = TYPE_LENGTH (type);
  regno = CALMRISC16_GR_REGNUM (0);
  off = 0;

  while (len > 0)
    {
      LONGEST val;
      int reglen = REGISTER_RAW_SIZE (regno);
      int vallen = len > reglen ? reglen : len;
      val = extract_signed_integer (valbuf + off, vallen);
      write_register (regno, val);
      len -= vallen;
      off += vallen;
      regno++;
    }
}

static void
calmrisc16_pop_frame_regular (struct frame_info *frame)
{
  int regno;
  CORE_ADDR fp;

  FRAME_INIT_SAVED_REGS (frame);
  
  /* Find the frame pointer value in order to properly restore the stack
     pointer.  */
  if (frame->extra_info->framereg == CALMRISC16_SP_REGNUM)
    fp = frame->frame;
  else
    {
      char *fr_raw_buffer = alloca (MAX_REGISTER_RAW_SIZE);
      int framereg = frame->extra_info->framereg;

      get_saved_register (fr_raw_buffer, NULL, NULL, frame, framereg, NULL);
      fp = extract_address (fr_raw_buffer, REGISTER_RAW_SIZE (framereg));
    }

  /* Restore the pc.  */
  write_register (pc_regnum, 
                  calmrisc16_insn_address_to_pointer (
		    calmrisc16_frame_saved_pc (frame)));

  /* Restore all other registers except for pc and sp.  */
  for (regno = 0; regno < CALMRISC16_NUM_REGS; regno++)
    {
      if (frame->saved_regs[regno] && regno != pc_regnum && regno != sp_regnum)
	{
	  write_register (regno,
	                  read_memory_integer (frame->saved_regs[regno],
			                       REGISTER_RAW_SIZE (regno)));
	}
    }

  /* Restore sp.  */
  write_register (sp_regnum, fp + frame->extra_info->frameoff);

  /* Forget about the frames that gdb has cached.  */
  flush_cached_frames ();
}

static void
calmrisc16_pop_frame (void)
{
  generic_pop_current_frame (calmrisc16_pop_frame_regular);
}

static void
calmrisc16_remote_translate_xfer_address (CORE_ADDR memaddr, int nr_bytes,
				   CORE_ADDR *targ_addr, int *targ_len)
{
  /* FIXME: Implement. */
  *targ_addr = memaddr;
  *targ_len  = nr_bytes;
}

/* The point of the pseudo-register stuff on CalmRisc16 is just to
   provide an alias for the link register.  This has two benefits;
   firstly, this makes it possible to refer to "a14" as "lr", secondly,
   we do some magic elsewhere so that "lr" will have the insn space
   mapping associated with it.  See calmrisc16_register_convert_to_virtual()
   and calmrisc16_register_convert_to_raw().  */
   
static void
calmrisc16_fetch_pseudo_register (int regno)
{
  if (regno == CALMRISC16_PSEUDO_LR_REGNUM)
    {
      char *buf = alloca (MAX_REGISTER_RAW_SIZE);
      CORE_ADDR lr_reg = read_register (CALMRISC16_LR_REGNUM);

      store_address (buf, REGISTER_RAW_SIZE (regno), lr_reg);
      supply_register (CALMRISC16_PSEUDO_LR_REGNUM, buf);
    }
}

static void
calmrisc16_store_pseudo_register (int regno)
{
  if (regno == CALMRISC16_PSEUDO_LR_REGNUM)
    {
      write_register (CALMRISC16_LR_REGNUM,
                      read_register (CALMRISC16_PSEUDO_LR_REGNUM));
    }
}

void
calmrisc16_get_saved_register (char *raw_buffer, 
                         int *optimized, 
			 CORE_ADDR *addrp,
			 struct frame_info *frame,
			 int regnum,
			 enum lval_type *lval)
{
  if (regnum == CALMRISC16_PSEUDO_LR_REGNUM)
    regnum = CALMRISC16_LR_REGNUM;

  generic_get_saved_register (raw_buffer, optimized, addrp, frame, regnum,
                              lval);
}

static void
register_info_string (int rn, char *buf)
{
  char *name = REGISTER_NAME (rn);
  char *name_colon = alloca (strlen (name) + 2);
  char *raw_buffer = alloca (MAX_REGISTER_RAW_SIZE);
  char *virtual_buffer = alloca (MAX_REGISTER_VIRTUAL_SIZE);
  int val_as_int;
  unsigned int val_as_addr;

  strcpy (name_colon, name);
  strcat (name_colon, ":");

  /* Get the data in raw format */
  read_relative_register_raw_bytes (rn, raw_buffer);

  /* Convert raw data to virtual format if necessary.  */
  if (REGISTER_CONVERTIBLE (rn))
    REGISTER_CONVERT_TO_VIRTUAL (rn, REGISTER_VIRTUAL_TYPE (rn),
				 raw_buffer, virtual_buffer);
  else
    memcpy (virtual_buffer, raw_buffer, REGISTER_VIRTUAL_SIZE (rn));

  val_as_int = (int) extract_signed_integer (virtual_buffer,
                                             REGISTER_VIRTUAL_SIZE (rn));
  val_as_addr = (unsigned int) extract_address (virtual_buffer,
                                                REGISTER_VIRTUAL_SIZE (rn));

  if ((CALMRISC16_GR_REGNUM_LO <= rn && rn <= CALMRISC16_GR_REGNUM_HI)
      || rn == CALMRISC16_SSR_FIQ_REGNUM 
      || rn == CALMRISC16_SSR_IRQ_REGNUM
      || rn == CALMRISC16_SSR_SWI_REGNUM)
    sprintf (buf, "%-9s0x%04x%7d", name_colon, val_as_addr, val_as_int);
  else if (CALMRISC16_ER_REGNUM_LO <= rn && rn <= CALMRISC16_ER_REGNUM_HI)
    sprintf (buf, "%-5s0x%02x", name_colon, val_as_addr);
  else if ((CALMRISC16_AR_REGNUM_LO <= rn && rn <= CALMRISC16_AR_REGNUM_HI)
	   || rn == CALMRISC16_SPC_FIQ_REGNUM
	   || rn == CALMRISC16_SPC_IRQ_REGNUM)
    sprintf (buf, "%-9s0x%06x", name_colon, val_as_addr);
  else if (rn == CALMRISC16_PSEUDO_LR_REGNUM)
    sprintf (buf, "%-4s0x%08x", name_colon, val_as_addr);
  else if (rn == CALMRISC16_SR_REGNUM)
    sprintf (buf,
             "%-9s0x%04x (T=%d, PM=%d, Z1=%d, Z0=%d, V=%d, TE=%d, IE=%d, FE=%d)",
	     name_colon, val_as_addr,
	     (val_as_addr >> 15) & 1,
	     (val_as_addr >> 6) & 1,
	     (val_as_addr >> 5) & 1,
	     (val_as_addr >> 4) & 1,
	     (val_as_addr >> 3) & 1,
	     (val_as_addr >> 2) & 1,
	     (val_as_addr >> 1) & 1,
	     val_as_addr & 1);
  else if (rn == CALMRISC16_PC_REGNUM)
    sprintf (buf, "%-5s0x%08x", name_colon, val_as_addr);
  else
    internal_error ("calmrisc16_registers_info: unhandled register %d\n", rn);
}

static void
calmrisc16_registers_info (int regno, int fpregs)
{
  char buf[128];

  if (regno >= 0)
    {
      register_info_string (regno, buf);
      printf_filtered ("%s\n", buf);
    }
  else
    {
      int i;
      for (i = 0; i < 16; i++)
	{
	  register_info_string (CALMRISC16_GR_REGNUM (i), buf);
	  if (i >= 8)
	    {
	      strcat (buf, "    ");
	      register_info_string (CALMRISC16_ER_REGNUM (i),
	                            buf + strlen (buf));
	      strcat (buf, "    ");
	      register_info_string (CALMRISC16_AR_REGNUM (i),
	                            buf + strlen (buf));
	      if (i == 14)
		{
		  strcat (buf, "    ");
		  register_info_string (CALMRISC16_PSEUDO_LR_REGNUM,
					buf + strlen (buf));
		}
	    }
	  printf_filtered ("%s\n", buf);
	}

      /* print ssr_fiq and spc_fiq */
      register_info_string (CALMRISC16_SSR_FIQ_REGNUM, buf);
      strcat (buf, "                 ");
      register_info_string (CALMRISC16_SPC_FIQ_REGNUM, buf + strlen (buf));
      printf_filtered ("%s\n", buf);

      /* print ssr_irq and spc_irq */
      register_info_string (CALMRISC16_SSR_IRQ_REGNUM, buf);
      strcat (buf, "                 ");
      register_info_string (CALMRISC16_SPC_IRQ_REGNUM, buf + strlen (buf));
      printf_filtered ("%s\n", buf);

      /* print ssr_swi */
      register_info_string (CALMRISC16_SSR_SWI_REGNUM, buf);
      printf_filtered ("%s\n", buf);

      /* print sr */
      register_info_string (CALMRISC16_SR_REGNUM, buf);
      printf_filtered ("%s\n", buf);

      /* print pc */
      register_info_string (CALMRISC16_PC_REGNUM, buf);
      printf_filtered ("%s\n", buf);
    }
}

int
calmrisc16_gcc_reg_to_regnum (int gccreg)
{
  /* See REGISTER_NAMES in GCC's config/calmrisc16/calmrisc16.h.  */
#if 0
  static int regmap[] = 
    { -1, CALMRISC16_GR_REGNUM (0), -1, -1,
      -1, CALMRISC16_GR_REGNUM (1), -1, -1,
      -1, CALMRISC16_GR_REGNUM (2), -1, -1,
      -1, CALMRISC16_GR_REGNUM (3), -1, -1,
      -1, CALMRISC16_GR_REGNUM (4), -1, -1,
      -1, CALMRISC16_GR_REGNUM (5), -1, -1,
      -1, CALMRISC16_GR_REGNUM (6), -1, -1,
      -1, CALMRISC16_GR_REGNUM (7), -1, -1,
      CALMRISC16_AR_REGNUM (8), CALMRISC16_GR_REGNUM (8), -1, -1,
      CALMRISC16_AR_REGNUM (9), CALMRISC16_GR_REGNUM (9), -1, -1,
      CALMRISC16_AR_REGNUM (10), CALMRISC16_GR_REGNUM (10), -1, -1,
      CALMRISC16_AR_REGNUM (11), CALMRISC16_GR_REGNUM (11), -1, -1,
      CALMRISC16_AR_REGNUM (12), CALMRISC16_GR_REGNUM (12), -1, -1,
      CALMRISC16_AR_REGNUM (13), CALMRISC16_GR_REGNUM (13), -1, -1,
      CALMRISC16_AR_REGNUM (14), CALMRISC16_GR_REGNUM (14), -1, -1,
      CALMRISC16_AR_REGNUM (15), CALMRISC16_GR_REGNUM (15), -1, -1,
      -1 /* t */, -1 /* Z0 */, -1 /* Z1 */, -1 /* AP */, -1 
    };
#else
  static int regmap[] = 
    {
      CALMRISC16_GR_REGNUM (0),
      CALMRISC16_GR_REGNUM (1),
      CALMRISC16_GR_REGNUM (2),
      CALMRISC16_GR_REGNUM (3),
      CALMRISC16_GR_REGNUM (4),
      CALMRISC16_GR_REGNUM (5),
      CALMRISC16_GR_REGNUM (6),
      CALMRISC16_GR_REGNUM (7),
      CALMRISC16_AR_REGNUM (8),
      CALMRISC16_AR_REGNUM (9),
      CALMRISC16_AR_REGNUM (10),
      CALMRISC16_AR_REGNUM (11),
      CALMRISC16_AR_REGNUM (12),
      CALMRISC16_AR_REGNUM (13),
      CALMRISC16_AR_REGNUM (14),
      CALMRISC16_AR_REGNUM (15)
    };
#endif

  if (gccreg < 0 || gccreg >= sizeof (regmap) / sizeof (regmap[0]) ||
      regmap[gccreg] == -1)
    {
      internal_error ("calmrisc16_gcc_reg_to_regnum: No mapping for %d\n",
		      gccreg);
    }

  return regmap[gccreg];
}

static struct gdbarch *
calmrisc16_gdbarch_init (struct gdbarch_info info, struct gdbarch_list *arches)
{
  struct gdbarch *gdbarch;

  /* There is only one calmrisc16 architecture.  */
  if (arches != NULL)
    return arches->gdbarch;
  gdbarch = gdbarch_alloc (&info, NULL);

  set_gdbarch_short_bit (gdbarch, 16);
  set_gdbarch_int_bit (gdbarch, 16);
  set_gdbarch_long_bit (gdbarch, 32);
  set_gdbarch_long_long_bit (gdbarch, 64);
  set_gdbarch_ptr_bit (gdbarch, 32);

  set_gdbarch_num_regs (gdbarch, CALMRISC16_NUM_REGS);
  set_gdbarch_num_pseudo_regs (gdbarch, CALMRISC16_NUM_PSEUDO_REGS);
  set_gdbarch_sp_regnum (gdbarch, sp_regnum);
  set_gdbarch_fp_regnum (gdbarch, fp_regnum);
  set_gdbarch_pc_regnum (gdbarch, pc_regnum);

  set_gdbarch_register_name (gdbarch, calmrisc16_register_name);
  set_gdbarch_register_size (gdbarch, 4);
  set_gdbarch_register_bytes (gdbarch, CALMRISC16_ALL_REGS * 4);
  set_gdbarch_register_byte (gdbarch, calmrisc16_register_byte);
  set_gdbarch_register_raw_size (gdbarch, calmrisc16_register_raw_size);
  set_gdbarch_max_register_raw_size (gdbarch, 4);
  set_gdbarch_register_virtual_size (gdbarch, calmrisc16_register_virtual_size);
  set_gdbarch_max_register_virtual_size (gdbarch, 4);
  set_gdbarch_register_virtual_type (gdbarch, calmrisc16_register_virtual_type);
  set_gdbarch_do_registers_info (gdbarch, calmrisc16_registers_info);
  set_gdbarch_fetch_pseudo_register (gdbarch, calmrisc16_fetch_pseudo_register);
  set_gdbarch_store_pseudo_register (gdbarch, calmrisc16_store_pseudo_register);

  set_gdbarch_skip_prologue (gdbarch, calmrisc16_skip_prologue);
  set_gdbarch_breakpoint_from_pc (gdbarch, calmrisc16_breakpoint_from_pc);

  set_gdbarch_frame_num_args (gdbarch, frame_num_args_unknown);
  set_gdbarch_frameless_function_invocation (gdbarch, calmrisc16_frameless_function_invocation);

  set_gdbarch_saved_pc_after_call (gdbarch, calmrisc16_saved_pc_after_call);

  set_gdbarch_frame_chain (gdbarch, calmrisc16_frame_chain);
  set_gdbarch_frame_chain_valid (gdbarch, func_frame_chain_valid);
  set_gdbarch_frame_saved_pc (gdbarch, calmrisc16_frame_saved_pc);
  set_gdbarch_frame_args_address (gdbarch, calmrisc16_frame_args_address);
  set_gdbarch_frame_locals_address (gdbarch, calmrisc16_frame_locals_address);

  set_gdbarch_frame_init_saved_regs (gdbarch, calmrisc16_frame_init_saved_regs);

  set_gdbarch_use_struct_convention (gdbarch, calmrisc16_use_struct_convention);
  set_gdbarch_extract_return_value (gdbarch, calmrisc16_extract_return_value);

  set_gdbarch_store_struct_return (gdbarch, calmrisc16_store_struct_return);
  set_gdbarch_store_return_value (gdbarch, calmrisc16_store_return_value);
  set_gdbarch_extract_struct_value_address (gdbarch, calmrisc16_extract_struct_value_address);

  set_gdbarch_read_pc (gdbarch, calmrisc16_read_pc);
  set_gdbarch_write_pc (gdbarch, calmrisc16_write_pc);

  set_gdbarch_register_convertible (gdbarch, calmrisc16_register_convertible);
  set_gdbarch_register_convert_to_virtual (gdbarch, calmrisc16_register_convert_to_virtual);
  set_gdbarch_register_convert_to_raw (gdbarch, calmrisc16_register_convert_to_raw);

  set_gdbarch_address_to_pointer (gdbarch, calmrisc16_address_to_pointer);
  set_gdbarch_pointer_to_address (gdbarch, calmrisc16_pointer_to_address);

  /* Settings for calling functions in the inferior.  */
  set_gdbarch_use_generic_dummy_frames (gdbarch, 1);
  set_gdbarch_call_dummy_length (gdbarch, 0);
  set_gdbarch_coerce_float_to_double (gdbarch, 
				      standard_coerce_float_to_double);
  set_gdbarch_push_arguments (gdbarch, calmrisc16_push_arguments);
  set_gdbarch_push_return_address (gdbarch, calmrisc16_push_return_address);
  set_gdbarch_pop_frame (gdbarch, calmrisc16_pop_frame);

  set_gdbarch_call_dummy_p (gdbarch, 1);
  set_gdbarch_call_dummy_words (gdbarch, calmrisc16_call_dummy_words);
  set_gdbarch_sizeof_call_dummy_words (gdbarch, sizeof (calmrisc16_call_dummy_words));
  set_gdbarch_call_dummy_breakpoint_offset_p (gdbarch, 1);
  set_gdbarch_init_extra_frame_info (gdbarch, calmrisc16_init_extra_frame_info);
  set_gdbarch_get_saved_register (gdbarch, calmrisc16_get_saved_register);
  set_gdbarch_save_dummy_frame_tos (gdbarch, generic_save_dummy_frame_tos);

  set_gdbarch_stab_reg_to_regnum (gdbarch, calmrisc16_gcc_reg_to_regnum);
  set_gdbarch_dwarf_reg_to_regnum (gdbarch, calmrisc16_gcc_reg_to_regnum);
  set_gdbarch_dwarf2_reg_to_regnum (gdbarch, calmrisc16_gcc_reg_to_regnum);
  set_gdbarch_ecoff_reg_to_regnum (gdbarch, calmrisc16_gcc_reg_to_regnum);
  set_gdbarch_sdb_reg_to_regnum (gdbarch, calmrisc16_gcc_reg_to_regnum);

  /* Settings that should be unnecessary.  */
  set_gdbarch_inner_than (gdbarch, core_addr_lessthan);

  set_gdbarch_read_fp (gdbarch, generic_target_read_fp);
  set_gdbarch_write_fp (gdbarch, generic_target_write_fp);
  set_gdbarch_read_sp (gdbarch, generic_target_read_sp);
  set_gdbarch_write_sp (gdbarch, generic_target_write_sp);

  set_gdbarch_call_dummy_location (gdbarch, AT_ENTRY_POINT);
  set_gdbarch_call_dummy_address (gdbarch, entry_point_address);
  set_gdbarch_call_dummy_breakpoint_offset (gdbarch, 0);
  set_gdbarch_call_dummy_start_offset (gdbarch, 0);
  set_gdbarch_pc_in_call_dummy (gdbarch, generic_pc_in_call_dummy);
  set_gdbarch_call_dummy_stack_adjust_p (gdbarch, 0);
  set_gdbarch_push_dummy_frame (gdbarch, generic_push_dummy_frame);
  set_gdbarch_fix_call_dummy (gdbarch, generic_fix_call_dummy);

  set_gdbarch_frame_args_skip (gdbarch, 0);

  set_gdbarch_decr_pc_after_break (gdbarch, 0);
  set_gdbarch_function_start_offset (gdbarch, 0);

  set_gdbarch_remote_translate_xfer_address (gdbarch, calmrisc16_remote_translate_xfer_address);

  return gdbarch;
}

void
_initialize_calmrisc16_tdep (void)
{
  register_gdbarch_init (bfd_arch_calmrisc16, calmrisc16_gdbarch_init);

  tm_print_insn = print_insn_calmrisc16;
}


/* Remove all this when we make a public release of the calmrisc16 port: */
/* Local variables: */
/* change-log-default-name: "ChangeLog.Cygnus" */
/* End: */
